/**
 * Reprezentace vazby mezi 2 neurony (first a second). 
 * @author Martin Kot
 * @version 1.0 2001-05-12
 */

public class Connection {
  /**Vaha*/
  private double weight;
  /**Zmena vahy v minulem kroku (pro setrvacnost)*/
  private double deltaWeight;
  /**Neuron nizsi vrstvy*/
  private Neuron first;
  /**Neuron vyssi vrstvy*/
  private Neuron second;

  /**
   * Konstruktor
   * @param first Neuron nizsi vrstvy
   * @param second Neuron vyssi vrstvy
   */
  Connection(Neuron first, Neuron second) {
    this.first = first;
    this.second = second;
  }

  /**
   * Slouzi k ziskani neuronu nizsi vrstvy
   * @return Neuron nizsi vrstvy
   */
  Neuron getFirst() {
    return first;
  }

  /**
   * Slouzi k ziskani neuronu vyssi vrstvy
   * @return Neuron vyssi vrstvy
   */
  Neuron getSecond() {
    return second;
  }

  /**
   * Slouzi k ziskani vahy spojeni
   * @return Vaha spojeni
   */
  public double getWeight() {
    return weight;
  }

  /**
   * Zvetseni vahy vazby
   * @param value Pricitana hodnota
   */
  void adjustWeight(double value) {
    this.deltaWeight = -1 * NeuralNet.ny * value + NeuralNet.mi * deltaWeight;
    this.weight += deltaWeight;
  }

  /**
   * Nastaveni nahodne pocatecni hodnoty vahy. Hodnota je v rozmezi 
   * <-maxWeight,-minWeight>U<minWeight,maxWeight>
   */
   public void initialize() {
     double rnd= Math.random();
     weight = (Math.random() * (NeuralNet.maxWeight - NeuralNet.minWeight) 
	+ NeuralNet.minWeight) * ((rnd<0.5)?-1:1);
   }


  /**
   * Predani signalu od prvniho neuronu k druhemu
   */
  void passSignal() {
    second.adjustPotential(weight * first.getState());
  }
}
