import java.awt.*;
import java.awt.event.*;
import java.util.*;
/**
 * Rozhrani k Backpropagation neuronove siti. Slouzi i jako spoustejici trida 
 * aplikace.
 * @author Martin Kot
 * @version 1.0 2001-05-15
 */

public class GUI extends Frame {
  /**Pole s vypisem chyby neuronove site*/
  private Label chybaValLabel = new Label();
  /**Pole pro vstupy neuronove site*/
  private TextField vstupField = new TextField();
  /**Navesti pro pole pro vystupy neuronove site*/
  private Label vystypLabel = new Label();
  /**Pole pro vystup neuronove site*/
  private TextField vystupField = new TextField();
  /**Navesti pole s vypisem chyby neuronove site*/
  private Label chybaLabel = new Label();
  /**Navesti pole s vypisem poctu opakovani trenovaci mnoziny*/
  private Label opakLabel = new Label();
  /**Pole s vypisem poctu opakovani trenovaci mnoziny*/
  private Label opakValLabel = new Label();
  /**Navesti pole vystupu neuronove site*/
  private Label vstupLabel = new Label();
  /**Tlacitko pro vyber vstupniho souboru*/
  private Button fileButton = new Button();
  /**Tlacitko pro zacatek a preruseni uceni site*/
  private Button ucitButton = new Button();
  /**Tlacitko pro vypocet vystupu k zadanym vstupum*/
  private Button vypocetButton = new Button();
  /**Tlacitko pro nove nahodne nastaveni vah*/
  private Button resetButton = new Button();
  /**Tlacitko pro nastaveni parametru site*/
  private Button setingButton = new Button();
  /**Label slouzici pro informace uzivateli*/
  private Label messageLabel = new Label();

  /**Pocet opakovani trenovaci mnoziny*/
  private int opakovani = 0;
  /**Parser vstupniho souboru*/
  private FileParser parser;
  /**Samotna sit*/
  private NeuralNet net;
  /**Okno pro nastavovani parametru site*/
  private Parametry parametry;
  /**Indikace, zda je sit ve stavu uceni nebo ne*/
  private boolean isLearning = false;
  /**Pocty neuronu v jednotlivych vrstvach*/
  private int[] neuronsCounts;

  /**
   * Konstruktor. Zaroven i okno zobrazi
   */
  public GUI() {
    super("Neuronova sit");
    net = new NeuralNet(new Observer(){
                 public void update(Observable o, Object arg){
                   showError(arg.toString());
                   opakovani++;
                   showRepeat();
		   repaint();
                 }  
    });
    parser = new FileParser();

    try {
      frameInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  /**
   * Zobrazi retezec v poli urcenem pro hodnotu chyby site.
   * @param s Zobrazovany retezec
   */
  private void showError(String s){
    this.chybaValLabel.setText(s);
  }

  /**
   * Zobrazi hodnotu tridni promenne opakovani v prislusnem poli
   */
  private void showRepeat(){
    this.opakValLabel.setText((new Integer(opakovani)).toString());
  }
  
  /**
   * Zobrazi chybovou hlasku v poli pro hlasky uzivateli
   * @param s Zobrazovana chyba
   */
  private void showBadMsg(String s){
    this.messageLabel.setText(s);
    this.messageLabel.setForeground(Color.red);
  }
  
  /**
   * Zobrazi hlasku v poli pro hlasky uzivateli
   * @param s Zobrazovana hlaska
   */
  private void showGoodMsg(String s){
    this.messageLabel.setText(s);
    this.messageLabel.setForeground(Color.green);
  }

  /**
   * Nastavi okno i jeho jednotlive prvky
   */
  private void frameInit() {
    this.setLayout(null);
    vstupLabel.setText("Vstupy");
    vstupLabel.setBounds(new Rectangle(10, 40, 70, 25));

    vstupField.setBounds(new Rectangle(10, 70, 270, 25));

    vystypLabel.setBounds(new Rectangle(10, 100, 70, 25));
    vystypLabel.setText("Vystupy");

    vystupField.setBounds(new Rectangle(10, 130, 270, 25));
    vystupField.setEditable(false);

    chybaLabel.setBounds(new Rectangle(10, 190, 140, 25));
    chybaLabel.setText("Chyba site");

    chybaValLabel.setBounds(new Rectangle(160, 190, 150, 25));
    chybaValLabel.setText("");

    opakLabel.setText("Opakovani mnoziny");
    opakLabel.setBounds(new Rectangle(10, 220, 140, 25));

    opakValLabel.setText("0");
    opakValLabel.setBounds(new Rectangle(160, 220, 150, 25));

    fileButton.setBounds(new Rectangle(315, 40, 120, 25));
    fileButton.setLabel("Soubor");
    fileButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        fileButtonAction(e);
      }
    });

    ucitButton.setLabel("Ucit");
    ucitButton.setEnabled(false);
    ucitButton.setBounds(new Rectangle(315, 70, 120, 25));
    ucitButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        ucitButtonAction(e);
      }
    });

    vypocetButton.setLabel("Vypocet");
    vypocetButton.setEnabled(false);
    vypocetButton.setBounds(new Rectangle(315, 100, 120, 25));
    vypocetButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        vypocetButtonAction(e);
      }
    });

    resetButton.setLabel("Reset site");
    resetButton.setEnabled(false);
    resetButton.setBounds(new Rectangle(315, 130, 120, 25));
    resetButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        resetButtonAction(e);
      }
    });

    setingButton.setLabel("Nastaveni");
    setingButton.setEnabled(true);
    setingButton.setBounds(new Rectangle(315, 160, 120, 25));
    setingButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        setingButtonAction(e);
      }
    });

    messageLabel.setBounds(new Rectangle(10, 250, 435, 25));

    this.setResizable(false);
    this.addWindowListener(new java.awt.event.WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        wClosed(e);
      }
    });
    this.setBounds(new Rectangle(100, 100, 455, 290));
    this.add(vstupField, null);
    this.add(vstupLabel, null);
    this.add(vystypLabel, null);
    this.add(vystupField, null);
    this.add(chybaLabel, null);
    this.add(opakLabel, null);
    this.add(chybaValLabel, null);
    this.add(opakValLabel, null);
    this.add(vypocetButton, null);
    this.add(fileButton, null);
    this.add(ucitButton, null);
    this.add(resetButton, null);
    this.add(setingButton, null);
    this.add(messageLabel, null);
    this.show();
  }

  /**
   * Ukonci program. Jde o akci pri zavirani okna.
   */
  private void wClosed(WindowEvent e) {
    System.exit(0);
  }

  /**
   * Akce tlacitka pro vyber souboru. Otevre dialog s vyberem jmena souboru.
   * Po jeho zadani soubor analyzuje a vytvori sit
   * @return false, pokud mel soubor spatny format nebo nebyl zadny zvolen
   */
  private boolean fileButtonAction(ActionEvent e) {

    showGoodMsg("");
    FileDialog fileSelect = new FileDialog(this, "Jmeno souboru site");
    fileSelect.show();
    if (fileSelect.getFile() != null) {
      if (fileSelect.getDirectory() != null){
        parser.setFileName(fileSelect.getDirectory() + fileSelect.getFile());
      }
      else {
        parser.setFileName(fileSelect.getFile());
      }
    }
    else {
      return false;
    }
    neuronsCounts = parser.getNeuronsCounts();
    if(neuronsCounts == null) {
      showBadMsg("Spatna specifikace topologie site.");
      return false;
    }
    else {
      net.createNet(neuronsCounts);
      net.initialize();
      setOnlyFile(true);
      showError("");
      opakovani=0;
      showRepeat();
      return true;
    }
  }
  
  /**
   * Znemozni/ umozni stisk tlacitek uceni, reset a vypocet
   * @param b true umozni stisk, false znemozni stisk
   */
  private void setOnlyFile(boolean b){
    vypocetButton.setEnabled(b);
    ucitButton.setEnabled(b);
    resetButton.setEnabled(b);
  }

  /**
   * Spusti v jinem vlakne uceni site. Znemozni stisk talcitek mimo konce 
   * uceni a nastaveni parametru. Po skonceni uceni zase umozni stisk tlacitek
   */
  private void learning(){
    isLearning=true;
    net.setEndLearning(false);
    this.showGoodMsg("Ucim se...");
    this.ucitButton.setLabel("Prerusit");
    this.fileButton.setEnabled(false);
    this.vypocetButton.setEnabled(false);
    this.resetButton.setEnabled(false);
    Runnable r = (new Runnable(){
       public void run(){
         net.learning(parser.getTrainIns(),parser.getTrainOuts());
         showGoodMsg("");
         ucitButton.setLabel("Ucit");
         fileButton.setEnabled(true);
         vypocetButton.setEnabled(true);
         resetButton.setEnabled(true);
    isLearning=false;
       }
    });
    new Thread(r).start();
  }

  /**
   * Akce pri stisku tlacitka uceni. Pokud se sit prave uci, ukonci uceni. 
   * Jinak uceni spusti.
   */
  private void ucitButtonAction(ActionEvent e) {
    showGoodMsg("");
    if(isLearning){
      net.setEndLearning(true);
    }
    else {
      this.learning();
    }
  }

  /**
   * Prevede retezec ve vstupnim poli na jednotlive realne hodnoty
   * @return Pole realnych hodnot vstupu site
   */
  private double[] parseInputs(){
    StringTokenizer t;
    double v[];
    int prvku;
    neuronsCounts = parser.getNeuronsCounts();
    if (neuronsCounts == null) {
      return null;
    }
    t = new StringTokenizer(vstupField.getText(), ",");
    prvku=neuronsCounts[0];

    if (t.countTokens()!=prvku) {
      return null;
    }
    v = new double[prvku];
    try {
      for(int i=0;i<prvku;i++){
        v[i]= new Double(t.nextToken().trim()).doubleValue();
      }
    }
    catch(NumberFormatException e) {
      return null;
    }
    return v;
  }

  /**
   * Zobrazeni vystupu neuronove site
   * @param outputs Pole realnych hodnot vystupu neuronove site
   * @return Pri nulove delce pole vraci false, jinak true
   */
  boolean showOutputs(double[] outputs){
    String vystup;
    if(outputs.length == 0){
      return false;
    }
    vystup = (new Float(outputs[0])).toString();
    for(int i = 1; i<outputs.length; i++){
      vystup += ", "+(new Float(outputs[0])).toString();
    }
    vystupField.setText(vystup);
    return true;
  }

  /**
   * Metoda pri stisku tlacitka vypocet. Rozlozi vstup, provede vypocet a 
   * zobrazi vystup.
   */
  void vypocetButtonAction(ActionEvent e) {
    showGoodMsg("");
    double[] vstupy = parseInputs();
    double[] vystupy;
    if(vstupy == null){
      showBadMsg("Spatne zadany vstupy");
    }
    else {
      vystupy = net.run(vstupy);
      showOutputs(vystupy);
    }       
  }


  /**
   * Akce pri stisku tlacitka reset. Provede znovunastaveni vah site na nahodne
   * hodnoty
   */
  void resetButtonAction(ActionEvent e) {
    showGoodMsg("");
    net.initialize();
    opakovani=0;
    showRepeat();
  }

  /**
   * Akce pri stisku tlacitka nastaveni. Zobrazi okno se vsemi parametry site.
   */
  void setingButtonAction(ActionEvent e) {
    showGoodMsg("");
    if(parametry == null){
      parametry = new Parametry(this);
    }
    parametry.refresh();
    hide();
    parametry.show();
  }

  /**
   * Spousteci metoda aplikace
   */
  public static void main(String argv[]){
    GUI gui = new GUI();
  }
}
