import java.util.*;

/**
 * Trida vazeb mezi neurony vrstvy
 * @author Martin Kot
 * @version 1.0 2001-05-12
 */

public class Interconnection {

  /**Vector vsech vazeb*/
  private Vector connections;


  /**
   * Konstruktor
   */
  Interconnection() {
    connections = new Vector();
  }

  /**
   * Prida novou vazbu
   * @param con Pridavana vazba
   */
  public void addConnection(Connection con) {
    connections.addElement(con);
  }


  /**
   * Ziskani horni vrstvy (neurony second) neuronu
   * @return vsechny neurony horni vrstvy
   */
  public Vector topNeurons() {
    Vector secNeurons = new Vector();
    Neuron neur;

    for (Enumeration e = connections.elements(); e.hasMoreElements(); ) {
      neur = ((Connection) e.nextElement()).getSecond();
      if (!secNeurons.contains(neur))
        secNeurons.add(neur);
    }
    return secNeurons;
  }
  
  /**
   * Ziskani dolni vrstvy neuronu (neurony first)
   * @return vsechny neurony dolni vrstvy
   */
  public Vector bottomNeurons() {
    Vector bottoms = new Vector();
    Neuron neur;

    for (Enumeration e = connections.elements(); e.hasMoreElements(); ) {
      neur = ((Connection) e.nextElement()).getFirst();
      if (!bottoms.contains(neur)) {
        bottoms.add(neur);
      }
    }
    return bottoms;
  }



  /**
   * Predani signalu od vstupnich neuronu k vystupnim
   */
  public void passSignal() {
    Enumeration e;
    Vector tops = topNeurons();

    //Vynuluje potencialy horni vrstvy neuronu
    for (e = tops.elements(); e.hasMoreElements(); ) {
      ((Neuron) e.nextElement()).resetPotential();
    }

    //Prenese potencial z dolni do horni vrstvy
    for (e = connections.elements(); e.hasMoreElements(); ) {
      ((Connection) e.nextElement()).passSignal();
    }

    //Vypocita excitaci neuronu horni vrstvy
    for (e = tops.elements(); e.hasMoreElements(); ) {
      ((Neuron) e.nextElement()).transfer();
    }
  }

  /**
   * Inicializace vah vsech vazeb na nahodnou hodnotu v intervalu
   * <-maxWeight,-minWeight>U<minWeight,maxWeight>
   */
  public void initialize() {
    for(Enumeration e=connections.elements();e.hasMoreElements();){
      Connection con = (Connection)e.nextElement();
      con.initialize();
    }
  }

  /**
   * Nastaveni novych vah vazeb vsech vazeb v interconnection podle chyby site
   */
  public void adjustWeights() {
    double d, y, x, delta;
    for (Enumeration e = connections.elements(); e.hasMoreElements(); ) { 
      Connection connection = (Connection) e.nextElement();
      d = connection.getSecond().getError();
      y = connection.getSecond().getState();
      x = connection.getFirst().getState();
      delta = d * y * (1 - y) * NeuralNet.slope * x;
      connection.adjustWeight(delta);
    }
  }


  /**
   * Spocita chybu vsech neuronu v interconnection
   */
  public void calculateError() {
    Connection connection;
    double d, y, yi, error;
    Neuron neuron;

    for (Enumeration e = bottomNeurons().elements(); e.hasMoreElements(); ) { 
      neuron = (Neuron) e.nextElement();
      error = 0;
      for (Enumeration en = connections.elements(); en.hasMoreElements(); ) {
        connection = (Connection) en.nextElement();
        if (neuron.equals(connection.getFirst())) {
          d = connection.getSecond().getError();
          y = connection.getSecond().getState();
          yi = connection.getFirst().getState();
          error += (d * y * (1-y) * NeuralNet.slope * connection.getWeight());
        }
      }
      neuron.setError(error);
    }
  }
}
