import java.util.*;
import java.text.*;

/**
 * Implementace neuronove site s topologii backpropagation
 * @author Martin Kot
 * @version 1.0 2001-05-12
 */
public class NeuralNet extends Observable{
  /**koeficient uceni*/
  public static double ny = 0.3; 
  /**koef. vlivu zmeny vah z minuleho kroku - setrvacnost site*/
  public static double mi = 0.05;
  /**maximalni chyba cele site, ktera je podminkou ukonceni uceni*/
  public static double maxError = 3.0E-4;
  /**pocet opakovani trenovaci mnoziny*/
  public static int repSet = 1000;
  /**pocet opakovani prvku trenovaci mnoziny pri jednom opakovani mnoziny*/
  public static int repItem = 5; 
  /**minimalni kladna hodnota vahy pri inicializaci*/
  public static double minWeight = 0.05;
  /**maximalni kladna hodnota vahy pri inicializaci*/
  public static double maxWeight = 0.3; 
  /**hodnota strmosti neuronu pri inicializaci*/
  public static double slope = 2.0; 
  
  /**Vsechna interconnection*/
  private Vector icons; 
  /**Vsechny neurony*/
  private Vector neurons; 
  /**Slouzi pro preruseni uceni jinym vlaknem*/ 
  private boolean endLearning = false;
  /**Chyba site*/
  private double error = 0;
  
  /**
   * Konstruktor
   * @param o Objekt zobrazujici chyby site pri uceni
   */
  public NeuralNet(Observer o) {
    addObserver(o);
    icons = new Vector();
    neurons = new Vector();
  }

  /**
   * Nastavi hodnotu promenne endLearning. Slouzi pro ukonceni uceni jinym 
   * vlaknem.
   * @param b Nastavovana hodnota. true znamena konec uceni.
   */
  public void setEndLearning(boolean b){
    this.endLearning=b;
  }
  
  /**
   *  Inicializace neuronove site. Inicializuje neurony i interconnection
   */
  public void initialize() {
    Enumeration e;
    
    for (e = neurons.elements(); e.hasMoreElements(); ) {
      ((Neuron)  e.nextElement()).initialize();
    }
    for (e = icons.elements(); e.hasMoreElements(); ) {
      ((Interconnection)  e.nextElement()).initialize();
    }
  }
  
  /**
   * Vytvoreni neuronove site
   * @param neuronsCounts Pocet neuronu ve vrstvach - prvni prvek je vstupni
   */
  public void createNet(int neuronsCounts[]) {
    icons = new Vector();
    neurons = new Vector();
    for (int i=0; i<neuronsCounts.length; i++) {
      createIcon(neuronsCounts[i]); 
    }
  }  
  
  /**
   *  Vytvoreni vrstvy
   *  @param neuronsCount pocet neuronu vrstvy
   */
  protected void createIcon(int neuronsCount) {
    Interconnection icon;
    Neuron neuron;
    Vector bottoms;
    Enumeration e;
    
    if (neurons.isEmpty() == false) { //=> nejde o 1. vrstvu 
      if(icons.isEmpty() == true) {  //=> jde o 2. vrstvu
        bottoms = (Vector) neurons.clone();
      }
      else {  //=>jde o vyssi vrstvu
        icon = (Interconnection) icons.lastElement(); 
        //horni neurony nizsi vrstvy budou dolnimi teto
        bottoms = ((Interconnection)icons.lastElement()).topNeurons();
      }
      
      icon = new Interconnection(); 
      for (int i = 0; i < neuronsCount; i++) {
          neurons.add(neuron = new Neuron());
          for (e = bottoms.elements(); e.hasMoreElements(); ) {
            icon.addConnection(new Connection((Neuron)e.nextElement(),neuron));
          }
      }
      icons.add(icon); 
    }
    else { //=> jde o 1. vrstvu
      for (int i = 0; i < neuronsCount; i++) {
          neurons.add(neuron = new Neuron()); //jen vytvori neurony, neni s cim
                                              //delat vazby
      }
    }
  }
  
  /**
   *  Uceni neuronove site
   *  @param trainIn Trenovaci mnozina - vstupy (Vector poli double cisel)
   *  @param trainOut Trenovaci mnozina - vystupy (Vector poli double cisel)
   *  @return True pri dokonceni uceni, pri preruseni vraci false.
   */
  public boolean learning(Vector trainIn, Vector trainOut) {
    double errors[] = new double[trainIn.size()]; 
    Vector outputNeurons = ((Interconnection)icons.lastElement()).topNeurons();
    for (int i = 0; i < repSet; i++) {
      for (int item = 0; item < trainIn.size(); item++) {
        double actItemIn[]= (double[]) trainIn.elementAt(item);
        double actItemOut[]= (double[]) trainOut.elementAt(item);
        for (int j = 0; j < repItem; j++) {
          if (endLearning) { 
            return false;
          }
          run(actItemIn); 
          adaptNeurons(actItemOut);
        }
         errors[item] = 0;
         for (int n = 0; n < outputNeurons.size(); n++) {
           errors[item] += (double) Math.pow(
		((Neuron) outputNeurons.get(n)).getError(),2); 
         }
      }
      error = 0;
      for (int k = 0; k < errors.length; k++) {
        error += errors[k];
      }
      error *= 0.5d;
      float flError=(new Double(error)).floatValue();
      
      //Vzhledem k problemum v linuxu se vypisuje chyba site i na konzoli
      System.out.println("Opakovani: "+i+"   chyba: "+flError);
      
      setChanged();
      notifyObservers(new Float(error));
      if (error <= maxError) {
        return true;
      }
    }
    return true;
  }

  /**
   * Provede adaptaci neuronove site
   * @param outputs Vystupy z trenovaci mnoziny. Jsou porovnavany s vypoctenymi
   * a podle toho je sit adaptovana
   */
  protected void adaptNeurons(double outputs[]) {
    double newError;
    Neuron outNeur;
    Vector outputNeurons = ((Interconnection)icons.lastElement()).topNeurons();
    for (int i = 0; i < outputNeurons.size(); i++) {
      outNeur=(Neuron) outputNeurons.get(i);
      newError=outNeur.getState()-((double []) outputs)[i];
      outNeur.setError(newError);
    }
    int size = icons.size();
    for (int i = size-1; i >= 0; i--) { 
      Interconnection interc=(Interconnection) icons.get(i);
      interc.calculateError();
      interc.adjustWeights();
    }
  }
  
  /**
   *  Vypocita vystup pro dany vstup
   *  @param inputs Pole vstupnich hodnot
   *  @return Pole vystupnich hodnot
   */
  public double[] run(double inputs[]) {
    int i=0;
    Enumeration e = ((Interconnection) icons.firstElement()).
		bottomNeurons().elements();
    for (; e.hasMoreElements(); i++) {
      Neuron neuron = (Neuron) e.nextElement();
      neuron.setPotential(inputs[i]);
      neuron.transfer();
    }

    for (e = icons.elements(); e.hasMoreElements(); ) {
      ((Interconnection) e.nextElement()).passSignal();
    }    
    return getOutputs();
  }
  
  /**
   *  Zjisteni vysledku po vypoctu z excitaci neuronu posledni vrstvy.
   *  @return vektor stavu vystupnich neuronu
   */
  public double[] getOutputs() {
    Vector outNeurs = ((Interconnection) icons.lastElement()).topNeurons();
    int outCount = outNeurs.size();
    double[] v = new double[outCount];
    for (int i=0; i<outCount; i++) {
      v[i]=((Neuron) outNeurs.get(i)).getState();
    }
    return v;
  }
}



